# -*- coding: utf-8 -*-
'''Yandex ID API wrapper

This module provides a wrapper for Yandex ID API.
Read more about Yandex ID API at https://yandex.ru/dev/id/doc/dg/api-id

Repository: https://github.com/LulzLoL231/yandexid
'''
from .yandexid.yandexid import YandexID
from .yandexid.async_yandexid import AsyncYandexID
from .yandexoauth.yandexoauth import YandexOAuth
from .yandexoauth.async_yandexoauth import AsyncYandexOAuth

from .__meta import __version__, __author__



# -*- coding: utf-8 -*-
'''Yandex ID avatar utilites
'''


def get_avatar_url(avatar_id: str, size: str = 'islands-200') -> str:
    '''Get avatar url by size

    Args:
        avatar_id (str): Avatar ID.
        size (str, optional): Avatar size. Default is `islands-200`.

    Note:
        size can be:\n
            - `islands-small` - 28x28;\n
            - `islands-34` - 34x34;\n
            - `islands-middle` - 42x42;\n
            - `islands-50` - 50x50;\n
            - `islands-retina-small` - 56x56;\n
            - `islands-68` - 68x68;\n
            - `islands-75` - 75x75;\n
            - `islands-retina-middle` - 84x84;\n
            - `islands-retina-50` - 100x100;\n
            - `islands-200` - 200x200;\n

    Returns:
        str: Avatar url
    '''
    return f'https://avatars.yandex.net/get-yapic/{avatar_id}/{size}'



# -*- coding: utf-8 -*-
'''Yandex ID OAuth API wrapper

Reference: https://yandex.ru/dev/id/doc/dg/oauth/concepts/about.html
'''
import logging
import warnings
from base64 import b64encode
from urllib.parse import urlencode

from httpx import Client

from ..__meta import __version__
from ..errors.yandexoauth import (
    InvalidDeviceID, InvalidDeviceName, AuthorizationPending,
    BadVerificationCode, InvalidClient, InvalidGrant, InvalidRequest,
    InvalidScope, UnauthorizedClient, UnsupportedGrantType, YandexOAuthError
)
from ..validators import (
    DeviceID, DeviceName, OptionalScope
)
from ..schemas.yandexoauth import Token


class YandexOAuth:
    '''Yandex ID OAuth API wrapper
    '''
    BASE_URL = 'https://oauth.yandex.ru'
    log = logging.getLogger('YandexOAuth')

    def __init__(
        self, client_id: str, client_secret: str, redirect_uri: str,
        scope: str | None = None, client: Client | None = None
    ):
        '''Initialize YandexOAuth object

        Args:
            client_id (str): Client ID
            client_secret (str): Client secret
            redirect_uri (str): Redirect URI
            scope (str, optional): OAuth Scope. Defaults to None.
            client (httpx.Client, optional): Client object. Defaults to None.
        '''
        self._client_id = client_id
        self._client_secret = client_secret
        self._redirect_uri = redirect_uri
        self._scope = scope

        self.__headers = {
            'User-Agent': f'YandexID/{__version__}'
        }
        self.__params = {
            'client_id': client_id
        }
        self.__client = client or Client(
            base_url=self.BASE_URL, params=self.__params,
            headers=self.__headers
        )

    def _make_request(self, method: str, url: str, **kwargs) -> dict:
        '''Make request to Yandex OAuth API

        Args:
            method (str): HTTP method
            url (str): URL to request

        Returns:
            dict: Response data
        '''
        response = self.__client.request(method, url, **kwargs)
        response.raise_for_status()
        return response.json()

    def get_authorization_url(
        self, response_type: str = 'code', device_id: str | None = None,
        device_name: str | None = None, login_hint: str | None = None,
        scope: str | None = None, optional_scope: str | None = None,
        force_confirm: bool | None = None, state: str | None = None
    ) -> str:
        '''Get authorization URL.
        Reference: https://yandex.ru/dev/id/doc/dg/oauth/reference/auto-code-client.html#auto-code-client__get-code

        Args:
            response_type (str, optional): Response type. Defaults to 'code'.
            device_id (str, optional): Device ID. Defaults to None.
            device_name (str, optional): Device name. Defaults to None.
            login_hint (str, optional): Login hint. Defaults to None.
            scope (str, optional): OAuth scope. Defaults to None.
            optional_scope (str, optional): Optional OAuth scope. Defaults to None.
            force_confirm (bool, optional): Force confirm. Defaults to None.
            state (str, optional): State. Defaults to None.

        Note:
            Check available `optional_scope` for your app here: https://oauth.yandex.ru/client/<client_id>/info.
            `response_type` can be 'code' or 'token'.

        Raises:
            InvalidDeviceID: If device_id is invalid.
            InvalidDeviceName: If device_name is invalid.

        Returns:
            str: Authorization URL
        '''
        params = {
            'response_type': response_type,
            'redirect_uri': self._redirect_uri,
            'client_id': self._client_id
        }
        if device_id:
            try:
                DeviceID.validate(device_id)
            except InvalidDeviceID as e:
                self.log.error(e)
                raise e
            else:
                params['device_id'] = device_id
                if not device_name:
                    warnings.warn(
                        'device_id is specified, but device_name is not. '
                        'Yandex ID returns token for unknown device.', UserWarning
                    )
        if device_name:
            try:
                DeviceName.validate(device_name)
            except InvalidDeviceName as e:
                self.log.error(e)
                raise e
            else:
                params['device_name'] = device_name
                if not device_id:
                    warnings.warn(
                        'device_name is specified, but device_id is not. '
                        'device_name will be ignored.', UserWarning
                    )
        if login_hint:
            params['login_hint'] = login_hint
        if scope:
            params['scope'] = scope
        if optional_scope:
            if self._scope or scope:
                if self._scope:
                    test_scope = self._scope
                elif scope:
                    test_scope = scope
                else:
                    test_scope = None
                if test_scope:
                    OptionalScope.validate(test_scope, optional_scope)
            params['optional_scope'] = optional_scope
        if force_confirm:
            params['force_confirm'] = str(int(force_confirm))
        if state:
            params['state'] = state
        return f'{self.BASE_URL}/authorize?{urlencode(params)}'

    def get_token_from_code(
        self, code: str, device_id: str | None = None,
        device_name: str | None = None
    ) -> Token:
        '''Get token from code
        Reference: https://yandex.ru/dev/id/doc/dg/oauth/reference/auto-code-client.html#auto-code-client__get-token

        Args:
            code (str): Verification code.
            device_id (str, optional): Device ID. Defaults to None.
            device_name (str, optional): Device name. Defaults to None.

        Raises:
            InvalidDeviceID: If device_id is invalid.
            InvalidDeviceName: If device_name is invalid.
            YandexOAuthError: If Yandex OAuth API returns error.

        Returns:
            Token: Token object
        '''
        url = '/token'
        method = 'POST'
        data = {
            'grant_type': 'authorization_code',
            'code': code,
        }
        if device_id:
            try:
                DeviceID.validate(device_id)
            except InvalidDeviceID as e:
                self.log.error(e)
                raise e
            else:
                data['device_id'] = device_id
                if not device_name:
                    warnings.warn(
                        'device_id is specified, but device_name is not. '
                        'Yandex ID returns token for unknown device.', UserWarning
                    )
        if device_name:
            try:
                DeviceName.validate(device_name)
            except InvalidDeviceName as e:
                self.log.error(e)
                raise e
            else:
                data['device_name'] = device_name
                if not device_id:
                    warnings.warn(
                        'device_name is specified, but device_id is not. '
                        'device_name will be ignored.', UserWarning
                    )
        headers = {
            'Authorization': f'Basic {b64encode(f"{self._client_id}:{self._client_secret}".encode()).decode()}'
        }
        response = self._make_request(method, url, data=data, headers=headers)
        if 'error' in response:
            match response['error']:
                case 'authorization_pending':
                    raise AuthorizationPending(response['error_description'])
                case 'bad_verification_code':
                    raise BadVerificationCode(response['error_description'])
                case 'invalid_client':
                    raise InvalidClient(response['error_description'])
                case 'invalid_grant':
                    raise InvalidGrant(response['error_description'])
                case 'invalid_request':
                    raise InvalidRequest(response['error_description'])
                case 'invalid_scope':
                    raise InvalidScope(response['error_description'])
                case 'unauthorized_client':
                    raise UnauthorizedClient(response['error_description'])
                case 'unsupported_grant_type':
                    raise UnsupportedGrantType(response['error_description'])
                case _:
                    raise YandexOAuthError(response['error_description'])
        return Token(**response)

    def get_token_from_refresh_token(self, refresh_token: str) -> Token:
        '''Get token from refresh token
        Reference: https://yandex.ru/dev/id/doc/dg/oauth/reference/refresh-client.html#refresh-client__get-token

        Args:
            refresh_token (str): Refresh token.

        Raises:
            YandexOAuthError: If Yandex OAuth API returns error.

        Returns:
            Token: Token object
        '''
        url = '/token'
        method = 'POST'
        data = {
            'grant_type': 'refresh_token',
            'refresh_token': refresh_token,
        }
        headers = {
            'Authorization': f'Basic {b64encode(f"{self._client_id}:{self._client_secret}".encode()).decode()}'
        }
        response = self._make_request(method, url, data=data, headers=headers)
        if 'error' in response:
            match response['error']:
                case 'invalid_client':
                    raise InvalidClient(response['error_description'])
                case 'invalid_grant':
                    raise InvalidGrant(response['error_description'])
                case 'invalid_request':
                    raise InvalidRequest(response['error_description'])
                case 'unauthorized_client':
                    raise UnauthorizedClient(response['error_description'])
                case 'unsupported_grant_type':
                    raise UnsupportedGrantType(response['error_description'])
                case _:
                    raise YandexOAuthError(response['error_description'])
        return Token(**response)

    def revoke_token(self, access_token: str) -> bool:
        '''Revoke token
        Reference: https://yandex.ru/dev/id/doc/dg/oauth/concepts/device-token.html#device-token-revoke__cleartext

        Args:
            access_token (str): OAuth access token.

        Raises:
            YandexOAuthError: If Yandex OAuth API returns error.

        Returns:
            bool: True if token was revoked successfully, False otherwise.
        '''
        url = '/revoke_token'
        method = 'POST'
        data = {
            'access_token': access_token
        }
        headers = {
            'Authorization': f'Basic {b64encode(f"{self._client_id}:{self._client_secret}".encode()).decode()}'
        }
        response = self._make_request(method, url, data=data, headers=headers)
        if 'error' in response:
            match response['error']:
                case 'invalid_client':
                    raise InvalidClient(response['error_description'])
                case 'invalid_request':
                    raise InvalidRequest(response['error_description'])
                case 'unauthorized_client':
                    raise UnauthorizedClient(response['error_description'])
                case _:
                    raise YandexOAuthError(response['error_description'])
        return True



# -*- coding: utf-8 -*-
'''JSON schemas for Yandex ID OAuth API
'''
from pydantic import BaseModel, Field


class Token(BaseModel):
    '''Token response

    Attributes:
        access_token (str): OAuth access token
        token_type (str): Token type. Must be always `bearer`.
        expires_in (int): Token expiration time in seconds
        refresh_token (str): Refresh token
        scope (str | None, optional): Scope of access token. Available only if some scope was declined.
    '''
    access_token: str = Field(..., description='OAuth access token')
    token_type: str = Field(..., description='Token type')
    expires_in: int = Field(..., description='Token expiration time in seconds')
    refresh_token: str = Field(..., description='Refresh token')
    scope: str | None = Field(None, description='Scope of access token')



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['yandexid',
 'yandexid.errors',
 'yandexid.schemas',
 'yandexid.yandexid',
 'yandexid.yandexoauth']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=2.6.0,<3.0.0', 'httpx>=0.23.3,<0.24.0', 'pydantic>=1.10.4,<2.0.0']

setup_kwargs = {
    'name': 'yandexid',
    'version': '0.3.1',
    'description': 'Yandex ID oauth API wrapper',
    'long_description': '# Яндекс ID (OAuth) API\nБиблиотека для работы с API Яндекс ID (OAuth) для Python 3.10+.\nПоддерживает асинхронную работу.\n\n[Документация API](https://yandex.ru/dev/id/doc/dg/index.html)\n\n## Установка\n\n1. С помощью pip:\n    \n    ```bash\n    pip install yandexid\n    ```\n\n2. С помощью pip+git:\n    \n    ```bash\n    pip install https://github.com/LulzLoL231/yandexid.git\n    ```\n\n3. Из исходников:\n\n    ```bash\n    git clone https://github.com/LulzLoL231/yandexid\n    pip install ./yandexid\n    ```\n\n## Пример использования\n\n1. Получение OAuth токена:\n\n    ```python\n    from yandexid import YandexOAuth\n\n    yandex_oauth = YandexOAuth(\n        client_id=\'<client_id>\',\n        client_secret=\'<client_secret>\',\n        redirect_uri=\'<redirect_uri>\'\n    )\n    auth_url = yandex_oauth.get_authorization_url()\n    # Тут нужно перейти по ссылке auth_url и получить код авторизации\n    token = yandex_oauth.get_token_from_code(\'<code>\')\n    ```\n    Возвращает объект `Token` с информацией о OAuth токене. Формат объекта совпадает с [форматом ответа из API Яндекс ID](https://yandex.ru/dev/id/doc/dg/oauth/reference/console-client.html#console-client__token-body-title).\n\n\n2. Получение информации о пользователе:\n\n    ```python\n    from yandexid import YandexID\n\n    yandex_id = YandexID(\'<oauth_token>\')\n    user_info = yandex_id.get_user_info_json()\n    ```\n    Возвращает объект `User` с информацией о пользователе. Формат объекта совпадает с [форматом ответа из API Яндекс ID](https://yandex.ru/dev/id/doc/dg/api-id/reference/response.html).\n\n## Асинхронная работа\nЧтобы использовать асинхронность, используйте классы `AsyncYandexOAuth` и `AsyncYandexID`:\n\n```python\nfrom yandexid import AsyncYandexID\n\nyandex_id = AsyncYandexID(\'<oauth_token>\')\nuser_info = await yandex_id.get_user_info_json()\n```\nНазвание методов полностью совпадает с названием синхронных методов, не забывайте использовать `await` перед вызовом асинхронных методов.\n\nЛоготипы Яндекс ID и название сервиса "Яндекс ID" принадлежат Яндексу.\n',
    'author': 'Maxim Mosin',
    'author_email': 'max@mosin.pw',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/LulzLoL231/yandexid',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
'''Yandex ID API data validators
'''
import warnings

from .errors.yandexoauth import (
    InvalidDeviceID, InvalidDeviceName
)


class DeviceID:
    '''Device ID validator
    '''
    @staticmethod
    def validate(device_id: str) -> bool:
        '''Validate device ID

        Args:
            device_id (str): Device ID

        Returns:
            bool: True if device ID is valid, False otherwise
        '''
        if len(device_id) < 6:
            raise InvalidDeviceID('Device ID is too short')
        if len(device_id) > 50:
            raise InvalidDeviceID('Device ID is too long')
        if not device_id.isalnum():
            raise InvalidDeviceID('Device ID must contain only alphanumeric characters')
        return True


class DeviceName:
    '''Device name validator
    '''
    @staticmethod
    def validate(device_name: str) -> bool:
        '''Validate device name

        Args:
            device_name (str): Device name

        Returns:
            bool: True if device name is valid, False otherwise
        '''
        if len(device_name) > 100:
            raise InvalidDeviceName('Device name is too long')
        return True


class OptionalScope:
    '''Optional scope validator
    '''
    @staticmethod
    def validate(scope: str, optional_scope: str):
        '''Validate optional scope

        Args:
            scope (str): OAuth scope
            optional_scope (str): Optional scope
        '''
        ignored_scope: list[str] = []
        for scope_item in optional_scope.split(','):
            if scope_item not in scope:
                ignored_scope.append(scope_item.strip())
        if ignored_scope:
            warnings.warn(
                f'Optional scopes {", ".join(ignored_scope)} is not in scope.',
                UserWarning
            )
